__all__ = ["PYTZ"]

import datetime
from typing import Literal

from dateutil.rrule import rrule

from ..cal import Timezone
from ..prop import vRecur
from .provider import TZProvider

class PYTZ(TZProvider):
    @property
    def name(self) -> Literal["pytz"]: ...
    def localize_utc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def localize(self, dt: datetime.datetime, tz: datetime.tzinfo) -> datetime.datetime: ...
    def knows_timezone_id(self, id: str) -> bool: ...
    def fix_rrule_until(self, rrule: rrule, ical_rrule: vRecur) -> None: ...
    def create_timezone(self, tz: Timezone) -> datetime.tzinfo: ...  # type: ignore[override]
    def timezone(self, name: str) -> datetime.tzinfo | None: ...
    def uses_pytz(self) -> bool: ...
    def uses_zoneinfo(self) -> bool: ...



import datetime
from _typeshed import Incomplete, Unused
from collections.abc import Iterator
from enum import Enum
from re import Pattern
from typing import Any, ClassVar, Final, TypeVar, overload
from typing_extensions import Self, TypeAlias

from .caselessdict import CaselessDict
from .parser import Parameters
from .parser_tools import ICAL_TYPE

__all__ = [
    "DURATION_REGEX",
    "TimeBase",
    "TypesFactory",
    "WEEKDAY_RULE",
    "tzid_from_dt",
    "vBinary",
    "vBoolean",
    "vCalAddress",
    "vCategory",
    "vDDDLists",
    "vDDDTypes",
    "vDate",
    "vDatetime",
    "vDuration",
    "vFloat",
    "vFrequency",
    "vGeo",
    "vInline",
    "vInt",
    "vMonth",
    "vPeriod",
    "vRecur",
    "vSkip",
    "vText",
    "vTime",
    "vUTCOffset",
    "vUri",
    "vWeekday",
]

_PropType: TypeAlias = type[Any]  # any of the v* classes in this file
_vRecurT = TypeVar("_vRecurT", bound=vRecur)

DURATION_REGEX: Final[Pattern[str]]
WEEKDAY_RULE: Final[Pattern[str]]

def tzid_from_dt(dt: datetime.datetime) -> str | None: ...

class vBinary:
    obj: Incomplete
    params: Parameters
    def __init__(self, obj) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical): ...
    def __eq__(self, other): ...

class vBoolean(int):
    BOOL_MAP: Incomplete
    params: Parameters
    def __new__(cls, *args, **kwargs): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vText(str):
    encoding: str
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8") -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vCalAddress(str):
    params: Parameters
    def __new__(cls, value, encoding="utf-8"): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vFloat(float):
    params: Parameters
    def __new__(cls, *args, **kwargs): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vInt(int):
    params: Parameters
    def __new__(cls, *args, **kwargs): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vDDDLists:
    params: Parameters
    dts: Incomplete
    def __init__(self, dt_list) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical, timezone: Incomplete | None = None): ...
    def __eq__(self, other): ...

class vCategory:
    cats: Incomplete
    params: Parameters
    def __init__(self, c_list) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> str: ...
    def __eq__(self, other: object) -> bool: ...

class TimeBase:
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self): ...

class vDDDTypes(TimeBase):
    params: Parameters
    dt: Incomplete
    def __init__(self, dt) -> None: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical, timezone: Incomplete | None = None): ...

class vDate(TimeBase):
    dt: Incomplete
    params: Parameters
    def __init__(self, dt) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical): ...

class vDatetime(TimeBase):
    dt: Incomplete
    params: Parameters
    def __init__(self, dt) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical, timezone: str | None = None) -> datetime.datetime: ...

class vDuration(TimeBase):
    td: Incomplete
    params: Parameters
    def __init__(self, td) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical): ...
    @property
    def dt(self): ...

class vPeriod(TimeBase):
    params: Parameters
    start: Incomplete
    end: Incomplete
    by_duration: Incomplete
    duration: Incomplete
    def __init__(self, per) -> None: ...
    def overlaps(self, other): ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical, timezone: Incomplete | None = None): ...
    @property
    def dt(self): ...

class vWeekday(str):
    week_days: Incomplete
    relative: Incomplete
    params: Parameters
    def __new__(cls, value, encoding="utf-8"): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vFrequency(str):
    frequencies: Incomplete
    params: Parameters
    def __new__(cls, value, encoding="utf-8"): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vMonth(int):
    leap: bool
    params: Parameters
    def __new__(cls, month: vMonth | str | int) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: vMonth | str | int) -> Self: ...

class vSkip(vText, Enum):
    OMIT = "OMIT"
    FORWARD = "FORWARD"
    BACKWARD = "BACKWARD"

    def __reduce_ex__(self, proto: Unused) -> tuple[Any, ...]: ...

class vRecur(CaselessDict[Incomplete]):
    frequencies: ClassVar[list[str]]
    canonical_order: ClassVar[tuple[str, ...]]
    types: ClassVar[CaselessDict[_PropType]]
    params: Parameters
    def __init__(self, *args, **kwargs) -> None: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def parse_type(cls, key, values): ...
    @classmethod
    @overload
    def from_ical(cls, ical: _vRecurT) -> _vRecurT: ...
    @classmethod
    @overload
    def from_ical(cls, ical: str) -> Self: ...

class vTime(TimeBase):
    dt: Incomplete
    params: Parameters
    def __init__(self, *args) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical): ...

class vUri(str):
    params: Parameters
    def __new__(cls, value, encoding="utf-8"): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class vGeo:
    latitude: Incomplete
    longitude: Incomplete
    params: Parameters
    def __init__(self, geo) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical): ...
    def __eq__(self, other): ...

class vUTCOffset:
    ignore_exceptions: bool
    td: Incomplete
    params: Parameters
    def __init__(self, td) -> None: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...
    def __eq__(self, other): ...

class vInline(str):
    params: Parameters
    def __new__(cls, value, encoding="utf-8"): ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical): ...

class TypesFactory(CaselessDict[_PropType]):
    all_types: tuple[_PropType, ...]
    def __init__(self, *args, **kwargs) -> None: ...
    types_map: CaselessDict[str]
    def for_property(self, name: str) -> _PropType: ...
    def to_ical(self, name: str, value) -> bytes: ...
    def from_ical(self, name: str, value): ...



from typing import Final

from .prop import vText

__all__ = ["UIDGenerator"]

class UIDGenerator:
    chars: Final[list[str]]
    @staticmethod
    def rnd_string(length: int = 16) -> str: ...
    @staticmethod
    def uid(host_name: str = "example.com", unique: str = "") -> vText: ...



__all__ = ["__version__", "version", "__version_tuple__", "version_tuple"]

from typing import Final

__version__: Final[str]
__version_tuple__: Final[tuple[int, ...]]
version: Final[str]
version_tuple: Final[tuple[int, ...]]



import datetime
from _typeshed import Incomplete, SupportsItems
from collections.abc import Callable
from typing import Any, ClassVar, Final, Literal, overload

from .caselessdict import CaselessDict
from .parser import Contentline, Contentlines
from .prop import TypesFactory

__all__ = [
    "Alarm",
    "Calendar",
    "Component",
    "ComponentFactory",
    "Event",
    "FreeBusy",
    "INLINE",
    "Journal",
    "Timezone",
    "TimezoneDaylight",
    "TimezoneStandard",
    "Todo",
    "component_factory",
    "get_example",
]

def get_example(component_directory: str, example_name: str) -> bytes: ...

class ComponentFactory(CaselessDict[Incomplete]):
    def __init__(self, *args, **kwargs) -> None: ...

INLINE: CaselessDict[int]

class Component(CaselessDict[Incomplete]):
    name: ClassVar[str | None]
    required: ClassVar[tuple[str, ...]]
    singletons: ClassVar[tuple[str, ...]]
    multiple: ClassVar[tuple[str, ...]]
    exclusive: ClassVar[tuple[str, ...]]
    inclusive: ClassVar[tuple[tuple[str, ...], ...]]
    ignore_exceptions: ClassVar[bool]
    subcomponents: list[Incomplete]
    errors: list[str]

    def __init__(self, *args, **kwargs) -> None: ...
    def __bool__(self) -> bool: ...
    __nonzero__ = __bool__
    def is_empty(self) -> bool: ...
    @overload
    def add(self, name: str, value: Any, *, encode: Literal[False]) -> None: ...
    @overload
    def add(self, name: str, value: Any, parameters: None, encode: Literal[False]) -> None: ...
    @overload
    def add(
        self, name: str, value: Any, parameters: SupportsItems[str, str | None] | None = None, encode: Literal[True] = True
    ) -> None: ...
    def decoded(self, name, default=[]): ...
    def get_inline(self, name, decode: bool = True): ...
    def set_inline(self, name, values, encode: bool = True) -> None: ...
    def add_component(self, component: Component) -> None: ...
    def walk(self, name: str | None = None, select: Callable[[Component], bool] = ...): ...
    def property_items(self, recursive: bool = True, sorted: bool = True): ...
    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[False] = False) -> Component: ...  # or any of its subclasses
    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[True]) -> list[Component]: ...  # or any of its subclasses
    def content_line(self, name: str, value, sorted: bool = True) -> Contentline: ...
    def content_lines(self, sorted: bool = True) -> Contentlines: ...
    def to_ical(self, sorted: bool = True) -> bytes: ...
    def __eq__(self, other: Component) -> bool: ...  # type: ignore[override]

class Event(Component):
    name: ClassVar[Literal["VEVENT"]]
    @classmethod
    def example(cls, name: str) -> Event: ...

class Todo(Component):
    name: ClassVar[Literal["VTODO"]]

class Journal(Component):
    name: ClassVar[Literal["VJOURNAL"]]

class FreeBusy(Component):
    name: ClassVar[Literal["VFREEBUSY"]]

class Timezone(Component):
    name: ClassVar[Literal["VTIMEZONE"]]
    @classmethod
    def example(cls, name: str) -> Calendar: ...
    def to_tz(self, tzp=...): ...  # FIXME -> DstTzInfo: ...
    @property
    def tz_name(self) -> str: ...
    def get_transitions(self) -> tuple[list[datetime.datetime], list[tuple[datetime.timedelta, datetime.timedelta, str]]]: ...

class TimezoneStandard(Component):
    name: ClassVar[Literal["STANDARD"]]

class TimezoneDaylight(Component):
    name: ClassVar[Literal["DAYLIGHT"]]

class Alarm(Component):
    name: ClassVar[Literal["VALARM"]]

class Calendar(Component):
    name: ClassVar[Literal["VCALENDAR"]]
    @classmethod
    def example(cls, name: str) -> Calendar: ...

types_factory: Final[TypesFactory]
component_factory: Final[ComponentFactory]



from _typeshed import SupportsItems
from collections import OrderedDict
from collections.abc import Iterable, Mapping
from typing import ClassVar, TypeVar, overload
from typing_extensions import Self

__all__ = ["canonsort_keys", "canonsort_items", "CaselessDict"]

_T = TypeVar("_T")
_VT = TypeVar("_VT")

def canonsort_keys(keys: Iterable[str], canonical_order: Iterable[str] | None = None) -> list[str]: ...
def canonsort_items(dict1: Mapping[str, _VT], canonical_order: Iterable[str] | None = None) -> list[tuple[str, _VT]]: ...

class CaselessDict(OrderedDict[str, _VT]):
    # Inherit complex __init__ from dict.
    def __getitem__(self, key: str | bytes) -> _VT: ...
    def __setitem__(self, key: str | bytes, value: _VT) -> None: ...
    def __delitem__(self, key: str | bytes) -> None: ...
    def __contains__(self, key: str | bytes) -> bool: ...  # type: ignore[override]
    @overload
    def get(self, key: str | bytes, default: None = None) -> _VT: ...
    @overload
    def get(self, key: str | bytes, default: _VT) -> _VT: ...
    @overload
    def get(self, key: str | bytes, default: _T) -> _VT | _T: ...
    @overload
    def setdefault(self: CaselessDict[_T | None], key: str | bytes, value: None = None) -> _T | None: ...
    @overload
    def setdefault(self, key: str | bytes, value: _VT) -> _VT: ...
    @overload  # type: ignore[override]
    def pop(self, key: str | bytes, default: None = None) -> _VT | None: ...
    @overload
    def pop(self, key: str | bytes, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: str | bytes, default: _T) -> _VT | _T: ...
    def popitem(self) -> tuple[str, _VT]: ...  # type: ignore[override]
    def has_key(self, key: str | bytes) -> bool: ...
    def update(self, *args: SupportsItems[str, _VT] | Iterable[tuple[str, _VT]], **kwargs: _VT) -> None: ...  # type: ignore[override]
    def copy(self) -> Self: ...
    def __eq__(self, other: SupportsItems[str, _VT]) -> bool: ...  # type: ignore[override]
    def __ne__(self, other: SupportsItems[str, _VT]) -> bool: ...  # type: ignore[override]
    canonical_order: ClassVar[Iterable[str] | None]
    def sorted_keys(self) -> list[str]: ...
    def sorted_items(self) -> list[tuple[str, _VT]]: ...



from .cal import (
    Alarm as Alarm,
    Calendar as Calendar,
    Component as Component,
    ComponentFactory as ComponentFactory,
    Event as Event,
    FreeBusy as FreeBusy,
    Journal as Journal,
    Timezone as Timezone,
    TimezoneDaylight as TimezoneDaylight,
    TimezoneStandard as TimezoneStandard,
    Todo as Todo,
)
from .parser import Parameters as Parameters, q_join as q_join, q_split as q_split
from .prop import (
    TypesFactory as TypesFactory,
    vBinary as vBinary,
    vBoolean as vBoolean,
    vCalAddress as vCalAddress,
    vDate as vDate,
    vDatetime as vDatetime,
    vDDDTypes as vDDDTypes,
    vDuration as vDuration,
    vFloat as vFloat,
    vFrequency as vFrequency,
    vGeo as vGeo,
    vInt as vInt,
    vPeriod as vPeriod,
    vRecur as vRecur,
    vText as vText,
    vTime as vTime,
    vUri as vUri,
    vUTCOffset as vUTCOffset,
    vWeekday as vWeekday,
)
from .timezone import use_pytz, use_zoneinfo
from .version import (
    __version__ as __version__,
    __version_tuple__ as __version_tuple__,
    version as version,
    version_tuple as version_tuple,
)

__all__ = [
    "Calendar",
    "Event",
    "Todo",
    "Journal",
    "Timezone",
    "TimezoneStandard",
    "TimezoneDaylight",
    "FreeBusy",
    "Alarm",
    "ComponentFactory",
    "vBinary",
    "vBoolean",
    "vCalAddress",
    "vDatetime",
    "vDate",
    "vDDDTypes",
    "vDuration",
    "vFloat",
    "vInt",
    "vPeriod",
    "vWeekday",
    "vFrequency",
    "vRecur",
    "vText",
    "vTime",
    "vUri",
    "vGeo",
    "vUTCOffset",
    "Parameters",
    "q_split",
    "q_join",
    "use_pytz",
    "use_zoneinfo",
    "__version__",
    "version",
    "__version_tuple__",
    "version_tuple",
    "TypesFactory",
    "Component",
]



from setuptools import setup

name = "types-icalendar"
description = "Typing stubs for icalendar"
long_description = '''
## Typing stubs for icalendar

This is a [PEP 561](https://peps.python.org/pep-0561/)
type stub package for the [`icalendar`](https://github.com/collective/icalendar) package.
It can be used by type-checking tools like
[mypy](https://github.com/python/mypy/),
[pyright](https://github.com/microsoft/pyright),
[pytype](https://github.com/google/pytype/),
PyCharm, etc. to check code that uses
`icalendar`.

This version of `types-icalendar` aims to provide accurate annotations
for `icalendar==6.0.*`.
The source for this package can be found at
https://github.com/python/typeshed/tree/main/stubs/icalendar. All fixes for
types and metadata should be contributed there.

See https://github.com/python/typeshed/blob/main/README.md for more details.
This package was generated from typeshed commit
[`8acc85dbabb93619291ecd1bd0271d5e1e55f9ee`](https://github.com/python/typeshed/commit/8acc85dbabb93619291ecd1bd0271d5e1e55f9ee) and was tested
with mypy 1.11.2, pyright 1.1.383, and
pytype 2024.9.13.
'''.lstrip()

setup(name=name,
      version="6.0.0.20241003",
      description=description,
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/python/typeshed",
      project_urls={
          "GitHub": "https://github.com/python/typeshed",
          "Changes": "https://github.com/typeshed-internal/stub_uploader/blob/main/data/changelogs/icalendar.md",
          "Issue tracker": "https://github.com/python/typeshed/issues",
          "Chat": "https://gitter.im/python/typing",
      },
      install_requires=['types-python-dateutil', 'backports.zoneinfo; python_version < "3.9"'],
      packages=['icalendar-stubs'],
      package_data={'icalendar-stubs': ['__init__.pyi', 'cal.pyi', 'caselessdict.pyi', 'parser.pyi', 'parser_tools.pyi', 'prop.pyi', 'timezone/__init__.pyi', 'timezone/provider.pyi', 'timezone/pytz.pyi', 'timezone/tzp.pyi', 'timezone/windows_to_olson.pyi', 'timezone/zoneinfo.pyi', 'tools.pyi', 'version.pyi', 'METADATA.toml', 'py.typed']},
      license="Apache-2.0",
      python_requires=">=3.8",
      classifiers=[
          "License :: OSI Approved :: Apache Software License",
          "Programming Language :: Python :: 3",
          "Typing :: Stubs Only",
      ]
)



import datetime
import sys
from typing import Final, Literal

from dateutil.rrule import rrule, rruleset

if sys.version_info >= (3, 9):
    from zoneinfo import ZoneInfo
else:
    from backports.zoneinfo import ZoneInfo

from ..cal import Timezone
from ..prop import vRecur
from .provider import TZProvider

__all__ = ["ZONEINFO"]

class ZONEINFO(TZProvider):
    @property
    def name(self) -> Literal["zoneinfo"]: ...
    utc: Final[ZoneInfo]
    def localize(self, dt: datetime.datetime, tz: ZoneInfo) -> datetime.datetime: ...  # type: ignore[override]
    def localize_utc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def timezone(self, name: str) -> datetime.tzinfo | None: ...
    def knows_timezone_id(self, id: str) -> bool: ...
    def fix_rrule_until(self, rrule: rrule, ical_rrule: vRecur) -> None: ...
    def create_timezone(self, tz: Timezone) -> datetime.tzinfo: ...  # type: ignore[override]
    def uses_pytz(self) -> Literal[False]: ...
    def uses_zoneinfo(self) -> Literal[True]: ...

def pickle_tzicalvtz(tzicalvtz): ...
def pickle_rruleset_with_cache(rs: rruleset): ...
def unpickle_rruleset_with_cache(rrule, rdate, exrule, exdate, cache): ...



from typing import Any, Final, TypeVar, overload
from typing_extensions import TypeAlias

_T = TypeVar("_T")

__all__ = ["DEFAULT_ENCODING", "SEQUENCE_TYPES", "ICAL_TYPE", "data_encode", "from_unicode", "to_unicode"]

SEQUENCE_TYPES: Final[tuple[type[Any], ...]]
DEFAULT_ENCODING: str
ICAL_TYPE: TypeAlias = str | bytes

def from_unicode(value: ICAL_TYPE, encoding: str = "utf-8") -> bytes: ...
def to_unicode(value: ICAL_TYPE, encoding: str = "utf-8-sig") -> str: ...
@overload
def data_encode(data: ICAL_TYPE, encoding: str = ...) -> bytes: ...
@overload
def data_encode(data: dict[Any, Any], encoding: str = ...) -> dict[Any, Any]: ...
@overload
def data_encode(data: list[Any] | tuple[Any, ...], encoding: str = ...) -> list[Any]: ...
@overload
def data_encode(data: _T, encoding: str = ...) -> _T: ...
